﻿local RIO = LibStub:GetLibrary("LibRaiderIO")

SLASH_RELOADUI1 = "/rl";
SlashCmdList["RELOADUI"] = ReloadUI;

SLASH_FRAMESTK1 = "/fs";
SlashCmdList["FRAMESTK"] = function()
	LoadAddOn("Blizzard_DebugTools");
	FrameStackTooltip_Toggle();
end

for i = 1, NUM_CHAT_WINDOWS do
	_G["ChatFrame"..i.."EditBox"]:SetAltArrowKeyMode(false)
end
	
local function GetRioScore(fullname, ...)
	local index = ...;
	
	if (not index) then index = 1 end;
	
	local raw = RIO.raw(1, strsplit("-", fullname));

	if (raw == nil) then return 0 end;
	
	local score = RIO.score(raw, index);

	return score;		
end

local function tablefind(tab,el)
    for index, value in pairs(tab) do
        if value == el then
            return index
        end
    end
end

local function trim(s)
   local from = s:match"^%s*()"
   return from > #s and "" or s:match(".*%S", from)
end

local function getRioScoreColor(rioScore)
	if (not rioScore) then return nil end;
	local color;
	if (rioScore <= 500) then
		color = "|cff80ff64+|r";
	elseif (rioScore <= 800) then
		color = "|cff5fba8e+|r";
	elseif (rioScore <= 1200) then
		color = "|cff6d5de5+|r"
	elseif (rioScore <= 1500) then
		color = "|cffae39e2+|r"
	elseif (rioScore <= 1800) then
		color = "|cffc443c5+|r"
	elseif (rioScore <= 2200) then
		color = "|cffdb529c+|r"
	elseif (rioScore <= 2500) then
		color = "|cffe75d80+|r"
	elseif (rioScore <= 2800) then
		color = "|cfff1695f+|r"
	elseif (rioScore <= 3000) then
		color = "|cfff6714a+|r"
	end
	return color;
end

local function getRioScoreText(rioScore, rioScoreOld, color)
	if (not rioScore) then return "" end;
	local rioText;
	if (rioScore == 0 and (not rioScoreOld or rioScoreOld == 0)) then 
		rioText = ""
	elseif (rioScoreOld and rioScore < 1000 and rioScore < rioScoreOld) then
		color = "|cffc0c0c0+|r"
		rioText = color:gsub("+", rioScoreOld).." ";
	else
		rioText = color:gsub("+", rioScore).." ";
	end
	return rioText;
end

local function getIndex(values, val)
	local index={};
	for k,v in pairs(values) do
	   index[v]=k;
	end
	return index[val];
end

function init(self, event, arg1)

	if (event == "ADDON_LOADED" and arg1 == "LFGFilter" ) then
		local function initDB(db, ...)
			local defaults = ...;
			if type(db) ~= "table" then db = {} end
			if type(defaults) ~= "table" then return db end
			for k, v in pairs(defaults) do
				if type(v) == "table" then
					db[k] = initDB(db[k], v)
				elseif type(v) ~= type(db[k]) then
					db[k] = v
				end
			end
			return db
		end

		LFGFilterSettings = initDB(LFGFilterSettings);

		self:UnregisterEvent("ADDON_LOADED");
	end	

	if (event == "PLAYER_LOGIN") then
		local function CreateButton(relativeFrame, typebutton, text, role, name, xPoint, yPoint)
	
			local isEnabled = LFGFilterSettings[name] or false;
			local button = CreateFrame("Button", nil, relativeFrame, "GameMenuButtonTemplate");
			local disabledTexture = "Interface\\Buttons\\UI-Panel-Button-Disabled";
			local enabledTexture = "Interface\\Buttons\\UI-Panel-Button-Up";
			
			local function setTexture(self, texture)
				self.Left:SetTexture(texture);
				self.Middle:SetTexture(texture);
				self.Right:SetTexture(texture);
			end
			
			local function setButton(btn, frame, enabled, ...)
				local filter, tFilter;
				local fromScript = ...;
				if (fromScript == true) then btn.enabled = not btn.enabled; else btn.enabled = enabled; end				
				if (btn.enabled == true) then
					setTexture(btn, enabledTexture);
					if (typebutton == "Filter") then					
						filter = trim(frame.filter.." "..role);	
					else
						filter = trim(frame.include.." "..role);
					end
					tFilter = { strsplit(" ", filter) };
				else
					setTexture(btn, disabledTexture);
					if (typebutton == "Filter") then					
						filter = trim(frame.filter:gsub(role, ""));
					else
						filter = trim(frame.include:gsub(role, ""));
					end					
					tFilter = { strsplit(" ", filter) };
				end
				
				if (typebutton == "Filter") then	
					frame.filter = filter;				
					LFGListFrame.SearchPanel.tFilter = tFilter;
				elseif (typebutton == "Include") then
					frame.include = filter;
					LFGListFrame.SearchPanel.tInclude = tFilter;
				end
			end
			
			button:SetPoint("LEFT", relativeFrame, "LEFT", xPoint, yPoint);
			button:SetSize(20, 20);
			button:SetText(text);
			button:SetNormalFontObject("GameFontNormalSmall");
			button:SetHighlightFontObject("GameFontHighlightSmall");			
			button:SetScript("OnShow", function(self) end);	
			button:SetScript("OnClick", function(self)
				setButton(self, relativeFrame, self.enabled, true);				
				LFGFilterSettings[name] = self.enabled;
			end);		
			
			setButton(button, relativeFrame, isEnabled);		
		
			return button;
		end
		
		local DF = CreateFrame("Frame", "DF_Frame", LFGListFrame.SearchPanel, "InsetFrameTemplate3");
		DF:SetSize(160, 120);
		DF:SetPoint("BOTTOMRIGHT", LFGListFrame.SearchPanel, "BOTTOMRIGHT", 0, -130);
		DF:SetMovable(true);
		DF:EnableMouse(true);
		DF:RegisterForDrag("LeftButton")
		DF:SetScript("OnDragStart", DF.StartMoving)
		DF:SetScript("OnDragStop", DF.StopMovingOrSizing)
		
		DF.filter = "";
		DF.include = "";

		DF.applyBtn = CreateFrame("Button", nil, DF, "GameMenuButtonTemplate");
		DF.applyBtn:SetPoint("RIGHT", DF, "RIGHT", -10, -40);
		DF.applyBtn:SetSize(50, 20);
		DF.applyBtn:SetText("Apply");
		DF.applyBtn:SetNormalFontObject("GameFontNormalSmall");
		DF.applyBtn:SetHighlightFontObject("GameFontHighlightSmall");	

		DF.tankFilterBtn = CreateButton(DF, "Filter", "T", "TANK", "TankFilterButton", 10, 40);
		DF.healerFilterBtn = CreateButton(DF, "Filter", "H", "HEALER", "HealerFilterButton",  40, 40);
		DF.damage1FilterBtn = CreateButton(DF, "Filter", "D", "DAMAGER", "Damager1FilterButton", 70, 40);
		DF.damage2FilterBtn = CreateButton(DF, "Filter", "D", "DAMAGER", "Damager2FilterButton", 100, 40);
		DF.damage3FilterBtn = CreateButton(DF, "Filter", "D", "DAMAGER", "Damager3FilterButton", 130, 40);

		DF.tankIncludeBtn = CreateButton(DF, "Include", "T", "TANK", "TankIncludeButton", 10, 10);	
		DF.healerIncludeBtn = CreateButton(DF, "Include", "H", "HEALER", "HealerIncludeButton", 40, 10);
		DF.damage1IncludeBtn = CreateButton(DF, "Include", "D", "DAMAGER", "Damager1IncludeButton", 70, 10);
		DF.damage2IncludeBtn = CreateButton(DF, "Include", "D", "DAMAGER", "Damager2IncludeButton", 100, 10);
		DF.damage3IncludeBtn = CreateButton(DF, "Include", "D", "DAMAGER", "Damager3IncludeButton", 130, 10);

		local function OnClickApply(self)		
			local filter = DF.filter;
			local tFilter = { strsplit(" ", filter) };
			local include = DF.include;
			local tInclude = { strsplit(" ", include) };
			
			DF.maxRioEdit:ClearFocus();
			DF.minRioEdit:ClearFocus();
			
			local minRio = DF.minRioEdit:GetNumber();
			local maxRio = DF.maxRioEdit:GetNumber();	
			LFGFilterSettings["minRioEdit"] = minRio;	
			LFGFilterSettings["maxRioEdit"] = maxRio;			
			LFGListFrame.SearchPanel.maxRio = maxRio or 0;
			LFGListFrame.SearchPanel.minRio = minRio or 0;		
				
			LFGListFrame.SearchPanel.tFilter = tFilter;	
			LFGListFrame.SearchPanel.tInclude = tInclude;
			
			LFGListSearchPanel_DoSearch(LFGListFrame.SearchPanel);
		end

		DF.applyBtn:SetScript("OnClick", OnClickApply);	
		
		DF.minRioEdit = CreateFrame("EditBox", nil, DF, "InputBoxInstructionsTemplate");
		DF.minRioEdit:SetAutoFocus(false);
		DF.minRioEdit:SetPoint("LEFT", DF, "LEFT", 40, -15);
		DF.minRioEdit:SetSize(50, 20);
		
		local minRioFromDB = LFGFilterSettings["minRioEdit"] or 0;
		if (minRioFromDB > 0) then
			DF.minRioEdit:SetText(minRioFromDB);
			LFGListFrame.SearchPanel.minRio = minRioFromDB;
		end

		DF.minRioEdit:SetScript("OnEnterPressed", function(self)
			self:ClearFocus();
			ChatFrame1EditBox:SetFocus();			
			LFGListFrame.SearchPanel.minRio = self:GetNumber();
			LFGFilterSettings["minRioEdit"] = self:GetNumber();
		end);
		
		DF.Label = DF:CreateFontString(nil , "BORDER", "GameFontNormal");
		DF.Label:SetJustifyH("CENTER");
		DF.Label:SetPoint("LEFT", DF, "LEFT", 10, -15);
		DF.Label:SetText("RIO");
		
		DF.maxRioEdit = CreateFrame("EditBox", nil, DF, "InputBoxInstructionsTemplate");
		DF.maxRioEdit:SetAutoFocus(false);
		DF.maxRioEdit:SetPoint("LEFT", DF, "LEFT", 100, -15);
		DF.maxRioEdit:SetSize(50, 20);
		
		local maxRioFromDB = LFGFilterSettings["maxRioEdit"] or 0;
		if (maxRioFromDB > 0) then 
			DF.maxRioEdit:SetText(maxRioFromDB);
			LFGListFrame.SearchPanel.maxRio = maxRioFromDB;
		end;	

		DF.maxRioEdit:SetScript("OnEnterPressed", function(self)
			self:ClearFocus();
			ChatFrame1EditBox:SetFocus();
			LFGListFrame.SearchPanel.maxRio = self:GetNumber();
			LFGFilterSettings["maxRioEdit"] = self:GetNumber();
		end);		
		
		DF.showRIO = CreateFrame("CheckButton", nil, DF, "UICheckButtonTemplate");
		DF.showRIO:SetPoint("LEFT", DF, "LEFT", 10, -40);
		DF.showRIO:SetSize(20, 20);
		DF.showRIO:SetScript("OnClick", function(self)
			local isChecked = self:GetChecked();
			LFGFilterSettings["showRIOChecked"] = isChecked;
			if (isChecked == true) then
				LFGListFrame.SearchPanel.showRIO = true;
			else
				LFGListFrame.SearchPanel.showRIO = false;
			end
			LFGListSearchPanel_UpdateResults(LFGListFrame.SearchPanel);
		end);
		
		local showRIOCheckedFromDB = LFGFilterSettings["showRIOChecked"] or false;
		LFGListFrame.SearchPanel.showRIO = showRIOCheckedFromDB;
		DF.showRIO:SetChecked(showRIOCheckedFromDB);

		DF.showClass = CreateFrame("CheckButton", nil, DF, "UICheckButtonTemplate");
		DF.showClass:SetPoint("LEFT", DF, "LEFT", 30, -40);
		DF.showClass:SetSize(20, 20);
		DF.showClass:SetScript("OnClick", function(self)
			local isChecked = self:GetChecked();
			LFGFilterSettings["showClassChecked"] = isChecked;
			LFGListFrame.SearchPanel.showClass = isChecked;
			LFGListSearchPanel_UpdateResults(LFGListFrame.SearchPanel);
		end);
		local showClassCheckedFromDB = LFGFilterSettings["showClassChecked"] or false;
		LFGListFrame.SearchPanel.showClass = showClassCheckedFromDB;
		DF.showClass:SetChecked(showClassCheckedFromDB);
		
		SLASH_DF1 = "/df";
		SlashCmdList["DF"] = function()
			if DF:IsShown() then
				DF:Hide()
			else
				DF:Show()
			end
		end	
		
		local function hook_LFGListSearchEntry_Update(entry, ...)
			local categoryID = LFGListFrame.SearchPanel.categoryID;
			local groups = LFGListFrame.SearchPanel.tGroups;
			local tResults = LFGListFrame.SearchPanel.results; 
			local resultID = entry.resultID;



			local totalResults = LFGListFrame.SearchPanel.totalResults;
			local tResults = LFGListFrame.SearchPanel.results;        
			local tFilter = LFGListFrame.SearchPanel.tFilter or {};
			local tInclude = LFGListFrame.SearchPanel.tInclude or {};
			local maxRio = LFGListFrame.SearchPanel.maxRio or 10000;
			local minRio = LFGListFrame.SearchPanel.minRio or 0;
			
			if ( maxRio == 0 ) then maxRio = 10000 end;
			
			LFGFilter_Filter(tResults, tFilter, tInclude, minRio, maxRio);

			for i = 1, 5 do
				local texture = "tex"..i;				
				if (entry.DataDisplay.Enumerate[texture]) then
					entry.DataDisplay.Enumerate[texture]:Hide();
				end				
			end
			
			if (LFGListFrame.SearchPanel.showClass == true and categoryID == 2) then			
				local numMembers = C_LFGList.GetSearchResultInfo(resultID).numMembers;
				local _, appStatus, pendingStatus, appDuration = C_LFGList.GetApplicationInfo(resultID);
				local isApplication = entry.isApplication;
				
				entry.DataDisplay:SetPoint("RIGHT", entry.DataDisplay:GetParent(), "RIGHT", 0, -5);
				
				--------------------------------------
				
				if ( pendingStatus == "applied" and C_LFGList.GetRoleCheckInfo() ) then
					entry.PendingLabel:SetText(LFG_LIST_ROLE_CHECK);
					entry.PendingLabel:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( pendingStatus == "cancelled" or appStatus == "cancelled" or appStatus == "failed" ) then
					entry.PendingLabel:SetText(LFG_LIST_APP_CANCELLED);
					entry.PendingLabel:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( appStatus == "declined" or appStatus == "declined_full" or appStatus == "declined_delisted" ) then
					entry.PendingLabel:SetText(LFG_LIST_APP_DECLINED);
					entry.PendingLabel:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( appStatus == "timedout" ) then
					entry.PendingLabel:SetText(LFG_LIST_APP_TIMED_OUT);
					entry.PendingLabel:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( appStatus == "invited" ) then
					entry.PendingLabel:SetText(LFG_LIST_APP_INVITED);
					entry.PendingLabel:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( appStatus == "inviteaccepted" ) then
					entry.PendingLabel:SetText(LFG_LIST_APP_INVITE_ACCEPTED);
					entry.PendingLabel:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( appStatus == "invitedeclined" ) then
					entry.PendingLabel:SetText(LFG_LIST_APP_INVITE_DECLINED);
					entry.PendingLabel:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				elseif ( isApplication and pendingStatus ~= "applied" ) then
					entry.PendingLabel:SetText(LFG_LIST_PENDING);
					entry.PendingLabel:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
					entry.PendingLabel:Show();
					entry.ExpirationTime:Show();
					entry.CancelButton:Show();
				else
					entry.PendingLabel:Hide();
					entry.ExpirationTime:Hide();
					entry.CancelButton:Hide();
				end
				
				--------------------------------
								
				local orderIndexes = {};
				
				for i=1, numMembers do					
					local role, class = C_LFGList.GetSearchResultMemberInfo(resultID, i);
					local orderIndex = getIndex(LFG_LIST_GROUP_DATA_ROLE_ORDER, role);
					table.insert(orderIndexes, {orderIndex, class});
				end
				
				table.sort(orderIndexes, function(a,b)
					return a[1] < b[1]
				end);
				
				local xOffset = -88;
				
				for i = 1, numMembers do
					local class = orderIndexes[i][2];
					local classColor = RAID_CLASS_COLORS[class];
					local r, g, b, a = classColor:GetRGBA();
					local texture = "tex"..i;
					
					if (not entry.DataDisplay.Enumerate[texture]) then
						entry.DataDisplay.Enumerate[texture] = entry.DataDisplay.Enumerate:CreateTexture(nil, "ARTWORK");
						entry.DataDisplay.Enumerate[texture]:SetSize(10, 3);
						entry.DataDisplay.Enumerate[texture]:SetPoint("RIGHT", entry.DataDisplay.Enumerate, "RIGHT", xOffset, 15);
					end
					
					entry.DataDisplay.Enumerate[texture]:Show();					
					entry.DataDisplay.Enumerate[texture]:SetColorTexture(r, g, b, 0.75);
					
					xOffset = xOffset + 18;					
				end
			end
			
			--------------------------------------
			
			if (groups) then
				for i, k in ipairs(groups) do	
					if (k.searchResultID == resultID) then				
						if (not k.leaderName) then						
							local leaderName = C_LFGList.GetSearchResultInfo(resultID).leaderName;
							k.leaderName = leaderName;
							if (leaderName) then
								k.rioScore = GetRioScore(leaderName);	
								k.rioScoreOld = GetRioScore(leaderName, 2);
								local maxRio = LFGListFrame.SearchPanel.maxRio or 10000;
								local minRio = LFGListFrame.SearchPanel.minRio or 0;
								
								if (k.rioScore ~= nil) then
									if ( k.rioScore > maxRio or k.rioScore < minRio or (k.rioScore == 0 and minRio > 0) ) then
										table.remove(tResults, tablefind(tResults, k["searchResultID"]));
									end
								end							
							end						
						end	
						
						if (LFGListFrame.SearchPanel.showRIO == true and k.rioScore) then
							local name = entry.Name:GetText();
							local color = getRioScoreColor(k.rioScore);
							local rioText = getRioScoreText(k.rioScore, k.rioScoreOld, color);			
							
							entry.Name:SetText(rioText..name);
						end
					end	
				end
			end
			
		end
		
		local function hook_LFGListApplicationViewer_UpdateApplicantMember(member, appID, memberIdx, ...)	
			local name = member.Name:GetText();
			local name, class, localizedClass, level, itemLevel, honorLevel, tank, healer, damage, assignedRole, relationship = C_LFGList.GetApplicantMemberInfo(appID, memberIdx);
			local rioScore = GetRioScore(name);
			local rioScoreOld = GetRioScore(name, 2);		
			local color = getRioScoreColor(rioScore);
			local rioText = getRioScoreText(rioScore, rioScoreOld, color);	
			
			if ( memberIdx > 1 ) then
				member.Name:SetText("  "..rioText..name);
			else
				member.Name:SetText(rioText..name);
			end
			
			local nameLength = 100;
			if ( relationship ) then
				nameLength = nameLength - 22;
			end
			if ( member.Name:GetWidth() > nameLength ) then
				member.Name:SetWidth(nameLength);
			end
		end
		
		hooksecurefunc("LFGListSearchEntry_Update", hook_LFGListSearchEntry_Update);
		hooksecurefunc("LFGListApplicationViewer_UpdateApplicantMember", hook_LFGListApplicationViewer_UpdateApplicantMember);	
		
		LFGListFrame.SearchPanel:HookScript("OnEvent", LFGFilter_OnEvent);		
		self:UnregisterEvent("PLAYER_LOGIN");
	end
	
	if (event == "PLAYER_LOGOUT") then
	end	
end

local function hasValue (tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            return true
        end
    end    
    return false
end

local function countValue (tab, val)
    local count = 0;
    for index, value in ipairs(tab) do
        if value == val then
            count = count + 1;
        end
    end    
    return count
end

function LFGFilter_Filter(t, f, inc, minRio, maxRio) 
    
    local tGroups = {};    
    
    if (t) then        
        for i, k in ipairs(t) do            
            local resultInfo = C_LFGList.GetSearchResultInfo(k);
            local activityId = resultInfo["activityID"];            
            local searchResultId = resultInfo["searchResultID"];
            local numMembers = resultInfo["numMembers"];
            local ilvl = resultInfo["requiredItemLevel"];			
			local leaderName = resultInfo["leaderName"];            
			
            local tRoles = {
                ["TANK"] = 0,
                ["HEALER"] = 0,
                ["DAMAGER"] = 0
            };
            
            local members = C_LFGList.GetSearchResultMemberCounts(searchResultId);            
            
            tRoles["TANK"] = members["TANK"];
            tRoles["HEALER"] = members["HEALER"];
            tRoles["DAMAGER"] = members["DAMAGER"];
            
            tGroups[i] = tRoles;
            tGroups[i].activityID = activityId;
            tGroups[i].searchResultID = searchResultId;
            tGroups[i].ilvl = ilvl;
            tGroups[i].numMembers = numMembers;
            tGroups[i].removed = false;
			tGroups[i].leaderName = leaderName;
			
			if (leaderName ~= nil and minRio ~= nil and maxRio ~= nil) then
				tGroups[i].rioScore = GetRioScore(leaderName);	
				tGroups[i].rioScoreOld = GetRioScore(leaderName, 2);				
			end
        end
        
        local filterTankCount = countValue(f, "TANK");
        local filterHealerCount = countValue(f, "HEALER");
        local filterDamagerCount = countValue(f, "DAMAGER");
        
        local includeTankCount = countValue(inc, "TANK");
        local includeHealerCount = countValue(inc, "HEALER");
        local includeDamagerCount = countValue(inc, "DAMAGER");
        
        for i, k in ipairs(tGroups) do
		
			if (k.rioScore ~= nil) then
				if ( k.rioScore > maxRio or k.rioScore < minRio or (k.rioScore == 0 and minRio > 0) ) then
					tGroups[i].removed = true;
				end
			end
            
            if ((k["TANK"] > 0 and filterTankCount > 0 and k["TANK"] <= filterTankCount )
                or (k["HEALER"] > 0 and filterHealerCount > 0 and k["HEALER"]<= filterHealerCount)
                or (k["DAMAGER"] > 0 and filterDamagerCount > 0 and k["DAMAGER"]<= filterDamagerCount )) then
                
                tGroups[i].removed = true;
            end
            
            if ((k["TANK"] ~= includeTankCount and includeTankCount > 0)
                or (k["HEALER"] ~= includeHealerCount and includeHealerCount > 0)
                or (k["DAMAGER"] ~= includeDamagerCount and includeDamagerCount > 0)) then
                
                tGroups[i].removed = true;
            end
            
            if (tGroups[i].removed == true) then
                table.remove(t, tablefind(t, k["searchResultID"])); 
            end
            
        end
		
		LFGListFrame.SearchPanel.tGroups = tGroups;
    end
end

function LFGFilter_OnEvent(self, event, ...)
    if ( event == "LFG_LIST_SEARCH_RESULTS_RECEIVED" ) then
		LFGListSearchPanel_UpdateResults(LFGListFrame.SearchPanel);	
    elseif ( event == "LFG_LIST_SEARCH_RESULT_UPDATED" ) then
		LFGListSearchPanel_UpdateResults(LFGListFrame.SearchPanel);
    elseif ( event == "LFG_LIST_SEARCH_FAILED" ) then
        print(event);    
    end
end

local events = CreateFrame("Frame");
events:RegisterEvent("ADDON_LOADED");
events:RegisterEvent("PLAYER_LOGIN");
events:RegisterEvent("PLAYER_LOGOUT");
events:SetScript("OnEvent", init);